module map_31
(map_out,
cpu_dat, cpu_addr, cpu_ce, cpu_rw, m2,
ppu_dat, ppu_addr, ppu_oe, ppu_we,
clk, map_cfg, rst);

	// Everdrive N8 inputs and outputs

	output [48:0]map_out;
	
	input [7:0]cpu_dat;
	input [14:0]cpu_addr;
	input cpu_ce, cpu_rw, m2;
	
	input [7:0]ppu_dat;
	input [13:0]ppu_addr;
	input ppu_oe, ppu_we;
	
	input clk, rst;
	input [7:0]map_cfg;
	
	wire ciram_a10, ciram_ce;
	wire [19:0]prg_addr;
	wire [19:0]chr_addr;
	wire prg_we;
	wire chr_we;
	wire irq;
	wire rom_ce;
	wire ram_ce;
	wire chr_ce;
	wire snd;
	
	assign map_out [48:0] = {
		snd, chr_ce, ram_ce, rom_ce, irq, chr_we, prg_we, ciram_ce, ciram_a10,
		chr_addr[19:0], prg_addr[19:0] };
	
	// Mapper
	
	reg [6:0] bank[0:7]; // 8 x 7 bit banking registers

	assign prg_we = 1'b0;
	assign ram_ce = 1'b0;
	assign rom_ce = !cpu_ce;
	assign chr_ce = ciram_ce;
	assign chr_we = map_cfg[7] ? !ppu_we & ciram_ce : 0; // allows 8k CHR ROM or RAM
	
	assign irq = 1'bz;
	assign snd = 1'bz;
	
	assign ciram_a10 = map_cfg[0] ? ppu_addr[10] : ppu_addr[11]; // mirroring
	assign ciram_ce = !ppu_addr[13];
	
	assign prg_addr[11:0] = cpu_addr[11:0];
	assign prg_addr[18:12] = bank[cpu_addr[14:12]][6:0];
	assign prg_addr[19] = 1'b0; // prg_addr[19] selects battery RAM, not used
	
	assign chr_addr[12:0] = ppu_addr[12:0];
	assign chr_addr[19:13] = 7'b0000000;
	
	initial
	begin
		// power on resets high bank
		bank[7][6:0] <= 7'b1111111;
	end

	always @(negedge m2)
	begin
		if (!cpu_rw & cpu_ce & (cpu_addr[14:12] === 3'b101))
		begin
			// writes to $5000-5FFF select a bank
			bank[cpu_addr[2:0]][6:0] <= cpu_dat[6:0];
		end
	end

endmodule
